<?php

/**
 * Copyright (c) 2023 schaal @it UG - info@schaal-24.de
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
*/

class installer_base {
	
	public function _exec($cmd) {

		$descriptorspec = array( 0 => array('pipe', 'r'), 1 => array('pipe', 'w'), 2 => array('pipe', 'w'));
		flush();
		$process = proc_open(exec($cmd, $ret, $val), $descriptorspec, $pipes, realpath('./'), array());
		if (is_resource($process)) {
			while ($s = fgets($pipes[1])) {
				print $s;
				flush();
			}
		}
		if($val != 0) die($cmd.' failed');
		return $ret;
	}

	public function free_query($query, $default, $name = '') {
		$this->swrite($query.' ['.$default.']: ');
		$input = $this->sread();
		$answer = @($input == '')?$default:$input;

		return $answer;
	}

	public function simple_query($query, $answers, $default, $name = '') {
		$finished = false;
		do {
			$answers_str = implode(',', $answers);
			$this->swrite($query.' ('.$answers_str.') ['.$default.']: ');
			$input = $this->sread();
			//* Select the default
			if($input == '') {
				$answer = $default;
				$finished = true;
			}
			//* Set answer id valid
			if(in_array($input, $answers)) {
				$answer = $input;
				$finished = true;
			}
		} while ($finished == false);
		
		return $answer;
	}

	public function sread() {
		$input = fgets(STDIN);
		return rtrim($input);
	}

	public function swrite($text = '', $mode = 'normal') {
		switch($mode) {
			case 'crit':
				echo substr($text,0, -1) . " - aborting\n";
				echo "\033[0;31";
				die();
				break;
			case 'warn':
				echo "\033[1;31m";
				break;
			case 'detail':
				echo "\033[1;32m";
				break;
			case 'info':
				echo "\033[0;32m";
				break;
			case 'note':
				echo "\033[0;34m";
				break;
			default:
				echo "\033[0m";
				 break;
		}
		echo $text;
		echo "\033[0m";
		if($mode == 'crit') exit;
	}

	public function swriteln($text = '', $mode = 'normal') {
		$this->swrite($text."\n", $mode);
	}

	public function validate($data, $type, $error) {
		switch($type) {
			case 'not_empty':
				if(trim($data) == '') {
					$this->swriteln($error . ' must not be empty!', 'warn');
					return false;
				}
				break;
			case 'is_ipv4':
				if (!filter_var($data, FILTER_VALIDATE_IP, FILTER_FLAG_IPV4)) {
					$this->swriteln($error . ' '.$data.' is not a valid IPv4 address!', 'warn');
					return false;
				}
				break;
			case 'netmask':
				if (!filter_var($data, FILTER_VALIDATE_INT, array('options' => array('min_range' => 8, 'max_range' => 29 )))) {
					$this->swriteln($error . ' '.$data.' is not a Subnet-Mask (8 - 29)', 'warn');
					return false;
				}
				break;
		}

		return true;
	}

	public function get_nic($retry=false) {
		global $inst, $install;

		if($install['nic'] == '') {
			$ret = $inst->_exec('ls /sys/class/net|grep -v lo');
			if(count($ret) !== 1) {
				if(!$ret) {
					$install['nic'] == '';
					return;
				}
				$check=false;
				do {
		 			$install['nic'] = $inst->simple_query('Multiple NICs found - choose your primary NIC', $ret, $ret[0], $install['nic']);
					if(in_array($install['nic'], $ret)) {
						$check = true;
					} 
				} while (!$check);
			}
			$install['nic'] = $ret[0];
		}
	}

	public function get_distname($short = false) {
		global $inst, $install;

		if(file_exists('/etc/os-release')) {
			$os = $this->ini_read('/etc/os-release');
			if(!isset($os['ID']) && isset($os['NAME'])) {
				$os['ID'] = strtolower(preg_replace('/^(\S*)\s*.*?$/', '$1', $os['NAME']));
			}
			if(!isset($os['ID']) || !$os['ID']) {
				$inst->swriteln('Detection of OS failed.', 'warn');
				return false;
			}
			if($os['ID'] !== 'debian' && $os['ID'] !== 'ubuntu') {
				$inst->swriteln($os['ID'] . ' is not a supported distribution.', 'warn');
				return false;
			}
			if(!isset($os['VERSION_ID']) && isset($os['VERSION'])) {
				$os['VERSION_ID'] = preg_replace('/^([0-9\.]+)(\s.*?)?$/', '$1', $os['VERSION']);
			}
			if(!isset($os['VERSION_ID']) || !$os['VERSION_ID']) {
				$inst->swriteln('Could not detect version of distribution ' . $os['ID'], 'warn');
				return false;
			}
			$os_data = array(
				'ID' => $os['ID'],
				'VERSION' => $os['VERSION_ID'],
				'NAME' => (isset($os['PRETTY_NAME']) ? $os['PRETTY_NAME'] : ucfirst($os['ID']) . ' ' . (isset($os['VERSION']) ? $os['VERSION'] : $os['VERSION_ID']))
			);
			if($short) {
				if($os['ID'] === 'ubuntu' && isset($os['UBUNTU_CODENAME'])) {
					return $os['UBUNTU_CODENAME'];
				}
				if($os['ID'] == 'debian' && isset($os['VERSION'])) {
					if(preg_match('/^\d+ \((\S+)\)/m', $os['VERSION'], $matches)) {
						if(isset($matches[1]) && $matches[1] != '') return $matches[1];
					}
				}
				return false;
			}
			return $os;
		}
		return false;
	}

	public function ini_read($file, $initial_sect = '', $has_variables = false) {
		if(!is_file($file)) {
			return false;
		}
		$fp = @fopen($file, 'r');
		
		$ini = array();
		$match = array();
		$sect = $initial_sect;
		$sect_data = array();
		if(!$fp) {
			return false;
		}
		while(!feof($fp)) {
			$line = trim(fgets($fp));
			if($line == '') {
				continue;
			} elseif(substr($line, 0, 1) === '#' || substr($line, 0, 1) === ';') {
				 // comments
				continue;
			} elseif(substr($line, 0, 2) === '//') {
				// comments
				continue;
			} elseif(preg_match('/^\s*\[(\w+)\]\s*$/', $line, $match)) {
				if($sect != '') {
					$ini[$sect] = $sect_data;
				}

				$sect = $match[1];
				$sect_data = array();
				continue;
			}

			if(strpos($line, '=') === false) {
				continue;
			} // invalid setting
			list($key, $value) = explode('=', $line);
			$key = trim($key);
			$value = trim($value);
			if($has_variables == true) {
				if(substr($key, 0, 1) === '$') {
					$key = substr($key, 1);
				}
				if(preg_match('/\w+\s*\[\s*(["\'])([^\]]*?)\\1\s*\]/', $key, $match)) {
					$key = $match[2];
				}
				if(substr($key, 0, 1) === "'" || substr($key, 0, 1) === '"') {
					$key = substr($key, 1, -1);
				}
				if(!$key) {
					continue;
				} // error or unsupported type
				
				$pos = strpos($value, '//');
				if($pos !== false) {
					$value = trim(substr($value, 0, $pos));
				}
				if(substr($value, -1) === ';') {
					$value = substr($value, 0, -1);
				}
				if(substr($value, 0, 1) === "'" || substr($value, 0, 1) === '"') {
					$value = substr($value, 1, -1);
				} elseif(!preg_match('/^[0-9](?:\.[0-9]*)$/', $value)) {
					 // not string and not numeric -> unsupported
					continue;
				}
			} elseif(preg_match('/^".*"$/', $value)) {
				$value = preg_replace('/^"(.*)"$/', '$1', $value);
			}
			$sect_data[$key] = $value;
		}
		fclose($fp);
		
		if($sect != '') {
			$ini[$sect] = $sect_data;
		} elseif($initial_sect == '') {
			return $sect_data;
		}

		return $ini;
	}

	public function api_credentials() {
		global $inst, $robot_account;

		$inst->swriteln('Enter your credentials for the Hetzner-API');
		foreach($robot_account as $t=>$i) {
			$check=false;
			do {
				$robot_account[$t] = $inst->free_query($t, $i);
				if($robot_account[$t] != '') $check = true;
			} while (!$check);
		}
		return $robot_account;
	}

	public function scan_files($dir, $only=array()) {
		$skip=array('.example');
		$files = array();

		$rii = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($dir));
		foreach ($rii as $file) {
			if ($file->isDir()) continue;
			if(!empty($only)) {
				foreach($only as $extension) {
					if($this->endsWith($file->getPathname(), $extension) === true && $ignore === false) {
						$files[] = $file->getPathname();
						continue;
					}
				}
			} else {
				if (!$this->endsWith($file->getPathname(), $skip)) $files[] = $file->getPathname();
			}
		}

		return $files;
	}

	public function endsWith($haystack, $needle) {

		$found = false;
		foreach($needle as $i) {
			$length = strlen($i);
			if( (substr($haystack, -$length) === $i)) return true;
		}

		return false;
	}

	public function replace_in_file($file) {
		global $install;
	
		if(file_exists('replace.conf.php')) {
			require('replace.conf.php');
		} else {
			$search = array('_EMAIL_');
			$replace = array ('email');
		}

		foreach($replace as $idx=>$val) {
			if(isset($install[$val])) $replace[$idx] = $install[$val];
		}

		$content = file_get_contents($file);
		$new_content = preg_replace($search, $replace, $content);
		return $new_content;
	}

	public function mkdir_custom($file) {
		$t = explode('/', $file);
		if (count($t) > 1) {
			array_pop($t);
			$new = implode('/', $t);
			exec(('mkdir -p '.$new), $ret, $val);
			if($ret == 0) return true; else return false;
		}
		return true;
	}

	public function disclaimer($title, $version) {
		$this->swriteln("Welcome to the $title Tool $version from schaal@it UG", 'info');
		$this->swriteln();
		$this->swriteln('You need to have some prerequisites set up to use this tool:');
		$this->swriteln(' * Credentials for the Hetzner-API');
		$this->swriteln();
		$this->swriteln('*** Disclaimer of Warranties ***', 'detail');
		$this->swriteln('schaal @it UG disclaims to the fullest extent authorized by law any and all other warranties, whether express or implied,');
		$this->swriteln('including, without limitation, any implied warranties of title, non-infringement, integration, merchantability or');
		$this->swriteln('fitness for a particular purpose.');
		$this->swriteln('By continuing to use this software, you agree to this.');
		$this->swriteln();
	}

	public function mkdirs($strPath, $mode = '0755') {
		if(isset($strPath) && $strPath != '') {
			if(is_dir($strPath)) return true;
			$pStrPath = dirname($strPath);
			if(!$this->mkdirs($pStrPath, $mode)) return false;
			$old_umask = umask(0);
			$ret_val = mkdir($strPath, octdec($mode));
			umask($old_umask);
			return $ret_val;
		}
		return false;
	}

	public function wf($file, $content) {
		$this->mkdirs(dirname($file));
		if(!$fp = fopen($file, 'wb')) $this->swriteln('WARNING: could not open file '.$file, 'crit');
		fwrite($fp, $content);
		fclose($fp);
	}
}
?>
